#ifndef BDFILE_H
#define BDFILE_H

/* 

    umesh2dgui, a GUI for umesh2d grid generator 

    Copyright (C) 2003 Gabriele Dini Ciacci
  
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
//#include "umesh2dgui.h"

#define PR_LINE 1
#define PR_CIRCLE 2
#define PR_ELLIPSE 3
#define PR_DATA 4


class Primitive {
 private:
 public:
  Primitive(){
    nextprimitive = NULL;
    prevprimitive = NULL;
    maxdata = 0;
    curdata = 0;
    data = NULL;
  }
  ~Primitive(){  }
  int id;
  int type;
  double *data;
  /* NOTE: a data element is a triplet of [i|s|h]data */
  int maxdata; // maximum number of data elements for witch mem is allcoated
  int curdata; // current number of data elements memorized
  double getdata(int x, int y); // y from 0 to 2
  void setdata(int x, int y, double value); // y from 0 to 2
  int allocatedata(int ndata);
  class Primitive *nextprimitive;
  class Primitive *prevprimitive;
};

class Line : public Primitive {
 public:
  Line() {
    type = PR_LINE;
  }
  ~Line(){  }
  int spline;
  double x1;
  double y1;
  double x2;
  double y2;
};

class Circle : public Primitive {
 public:
  Circle() {
    type = PR_CIRCLE;
  }
  ~Circle() {  }
  int spline;
  double radius;
  double centerx;
  double centery;
  int startarc;
  int endarc;
};

class Ellipse : public Primitive {
 public:
  Ellipse() {
    type = PR_ELLIPSE;
  }
  ~Ellipse() {  }
  int spline;
  double semix;
  double semiy;
  double centerx;
  double centery;
  int startarc;
  int endarc;
};

class Data : public Primitive {
};

class BDFile {
public:
  BDFile();
  ~BDFile() {  }

  /* PRIMITIVE MANGEMENT */
  Primitive *firstprimitive; //temporary, write a getfirstprimitive() function
  Primitive *add_prim(int type);
  void del_prim(class Primitive *oldprim);

  /* FILE GENERATION */
  int elements; //temporary, write a getprimn() function

  /* FILE OUTPUT */
  int outputbfile();
  int outputdfile();
  /* make a function that manages the file buffers internaly */
  char bfilename[256]; //Must allready contain rigth boundary.name
  char dfilename[256]; //Must allready contain rigth domain.name

private:
  /* PRIMITIVE MANGEMENT */

  Primitive *lastprimitive;
  Primitive *new_prim(int type);
  int globalprimid;  
  // in the future the id of the prim need to be managed much better.

  /* FILE GENERATION */
  inline void printhash(FILE *output);
  inline void printgeom(FILE *output);
  inline void printtopo(FILE *output);
  inline void printnvne(FILE *output);
  void printverelem(FILE *output);
  inline void printish(FILE *output);
  inline void printenbvev(FILE *output);
  void printprimg(FILE *output, Primitive *curprim);
  void printprimd(FILE *outptu, Primitive *curprim);
  void printbegin(FILE *output, int id);
  inline void printend(FILE *output);
  void printpriminfo(FILE *output);
  void printprimdata(FILE *output);
  int vertexs;


  /* FILE OUTPUT */
  int openbfile();
  int opendfile();
  void addboundary(char *buffer);
  FILE *bfile;
  FILE *dfile;

};

#endif  /* BDFILE_H */
