/*   umesh2dgui, a GUI for umesh2d grid generator 

    Copyright (C) 2003 Gabriele Dini Ciacci
  
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "umesh2dgui.h"

/* GLOBALS */
MyGL *mygl;
BDFile *bdfile;
fltk::ScrollGroup *datagroup;
fltk::Group *subwindow;
fltk::Window *window;
int inputindex[10];
int filenameinput_i; //trailing _i is for index
int ydataposition = 0;

int main(){

  /* PRINT OF SOME ABOUT INFO TO STDOUT */
  fprintf(stdout, "%s%s%s by %sGabriele Dini Ciacci%s - version %s%s%s\n", \
  ASC_RED, PACKAGE, ASC_RESET, ASC_GREEN, ASC_RESET, ASC_BLUE, VERSION, ASC_RESET);

  /* CLASS DECLARATION */
  bdfile = new BDFile;
  window = new fltk::Window(640, 510);

  /* WINDOW SETUP */
  window->begin();
  fltk::MenuBar *menubar = new fltk::MenuBar(0, 0, 640, 30, 0);
  fltk::TabGroup *tabbar = new fltk::TabGroup(10, 30, 620, 480, 0);
  window->end();

  /* TAB BAR SETUP */
  tabbar->begin();
  fltk::Group *boundarygroup = new fltk::Group(10, 30, 620, 450, "Boundary");
  fltk::Group *domaingroup =  new fltk::Group(10, 30, 620, 450, "Domain");
  tabbar->end();

  /* TITLE BOX SETUP */ /* removed now */
  /*  fltk::Widget *box = new fltk::Widget(10, 30, 620, 40, "umesh2dgui");
   *  box->box(fltk::UP_BOX);
   *  box->labelfont(fltk::HELVETICA_BOLD_ITALIC);
   *  box->labelcolor(fltk::GREEN);
   *  box->labelsize(40);
   *  box->labeltype(fltk::SHADOW_LABEL);
   */

  /* BOUNDARY TAB */
  boundarygroup->begin();
  fltk::Choice *primitivemenu = new fltk::Choice(200, 10, 100, 20, "Choose type of primitive to add:");
  subwindow = new fltk::Group(10, 50, 620, 320, "Insert primitive data here:");
  fltk::Input *filenameinput = new fltk::Input(80, 380, 250, 20, "Filename :");
  filenameinput_i = window->find(filenameinput);
  fltk::Button *filenamebrowser = new fltk::Button(332, 380, 20, 20, "B");
  filenamebrowser->callback((fltk::Callback *)openfilebrowser);
  //  filenamebrowser->begin();
  //  fltk::Pixmap pixmap = new fltk::Pixmap(folder03_xpm);
  //  filenamebrowser->end();
  fltk::Button *createfilebutton = new fltk::Button(360, 380, 100, 20, "Create");
  createfilebutton->callback((fltk::Callback *)createfile);

  /* PRIMITIVE MENU */
  //  primitivemenu->add("&clear", 0, (fltk::Callback *)clear_cb, 0, 0);
  primitivemenu->add("&line", 0, (fltk::Callback *)line_cb, 0, 0);
  primitivemenu->add("&circle", 0, (fltk::Callback *)circle_cb, 0, 0);
  primitivemenu->add("&ellipse", 0, (fltk::Callback *)ellipse_cb, 0, 0);
  primitivemenu->add("&data", 0, (fltk::Callback *)clear_cb, 0, 0);

  /* OPENGL WINDOW */
  mygl = new MyGL::MyGL(460, 5, 70, 70, 0);

  boundarygroup->end();

  /* DOMAIN TAB */
  domaingroup->begin();
  /* PLACEHOLDER */
  domaingroup->end();

  /* MENUBAR SETUP */
  menubar->begin();
  /* view menu */
  fltk::ItemGroup *view = new fltk::ItemGroup("&View");
  view->begin();
  fltk::Item *glview = new fltk::Item("&Toggle GL view");
  glview->callback((fltk::Callback *)glview_cb);
  view->end();
  /* help menu */
  fltk::ItemGroup *help = new fltk::ItemGroup("&Help");
  help->begin();
  fltk::Item *about = new fltk::Item("&About umesh2dgui");
  about->callback((fltk::Callback *)about_cb);
  help->end();
  menubar->end();

  /* LOOP*/
  window->show();
  subwindow->show();
  return fltk::run();
}

void openfilebrowser() {
  ((fltk::Input *)window->child(filenameinput_i))->value(fltk::file_chooser("Select a file", NULL, NULL)); 
  /* try to call file browser several time. Value() wants a good static buffer ot what?
   * LOOK LIKE WORKING... but i'm really not sure if will under extreme condition
   */
}

void createfile() {
  int returnvalue;
  strncpy(bdfile->bfilename,((fltk::Input *)window->child(filenameinput_i))->value(), 256);
  returnvalue = bdfile->outputbfile();
  if(returnvalue < 0) {
    fprintf(stderr, "createfile: Error creating the file\n");
    if(returnvalue == -1) fltk::alert("Error creating file: Invalid filename");
    if(returnvalue == -2) fltk::alert("Error creating file: File opening was not possible");
  }
}
void line_cb(){
  /* POPULATE THE SUBWINDOW */
  clear_cb();
  subwindow->begin();
  /* spline*/
  fltk::ValueInput *splinepoints = new fltk::ValueInput(200, 10, 100, 20, "N. points for spline aprox.: ");
  splinepoints->range(0, 10000);
  splinepoints->linesize(1.0);
  splinepoints->step(1);
  //  splinepoints->pagesize(10.0);
  inputindex[0] = subwindow->find(splinepoints);
  //  printf("%i", inputindex[0]);
  /* x1 */
  fltk::ValueInput *x1 = new fltk::ValueInput(200, 30, 100, 20, "Start point x coordinate: ");
  x1->range(-1000, 1000);
  x1->step(0.01);
  x1->linesize(0.01);
  //  x1->pagesize(1.0);
  inputindex[1] = subwindow->find(x1);
  //  printf("%i",  inputindex[1]);
  /* y1 */
  fltk::ValueInput *y1 = new fltk::ValueInput(450, 30, 100, 20, "Start point y coordinate: ");
  y1->range(-1000, 1000);
  y1->step(0.01);
  y1->linesize(0.01);
  //  y1->pagesize(1.0);
  inputindex[2] = subwindow->find(y1);
  //  printf("%i", inputindex[2]);
  /* x2 */
  fltk::ValueInput *x2 = new fltk::ValueInput(200, 50, 100, 20, "End point x coordinate: ");
  x2->range(-1000, 1000);
  x2->step(0.01);
  x2->linesize(0.01);
  //  x2->pagesize(1.0);
  inputindex[3] = subwindow->find(x2);
  //  printf("%i", inputindex[3]);
  /* y2 */
  fltk::ValueInput *y2 = new fltk::ValueInput(450, 50, 100, 20, "End point y coordinate: ");
  y2->range(-1000, 1000);
  y2->step(0.01);
  y2->linesize(0.01);
  //  y2->pagesize(1.0);
  inputindex[4] = subwindow->find(y2);
  //  printf("%i", inputindex[4]);
  /* [i|s|h]data */
  fltk::Button *new_dataline = new fltk::Button(0, 80, 580, 20, "Press here to add a new data entry");
  new_dataline->callback((fltk::Callback *)new_dataline_cb);
  datagroup = new fltk::ScrollGroup(0, 120, 580, 160, "Insert [i|s|h]data here:");
  datagroup->scrollbar.linesize(20);
  //  datagroup->begin();
  //  fltk::Widget *box2 = new fltk::Widget(0, 0, 620, 160, "TEXT"); // this is to see the space avaiable
  //  fltk::Scroll *datascroll = new fltk::Scroll(600, 0, 20, 160, 0);
  //  datagroup->end();
  /* buttons */
  fltk::Button *cancel = new fltk::Button(200, 290, 100, 20, "Cancel");
  fltk::Button *add = new fltk::Button(310, 290, 100, 20, "Add");
  cancel->callback((fltk::Callback *)clear_cb);
  add->callback((fltk::Callback *)add_line_cb);
  subwindow->end();
  subwindow->redraw(0, 0, 620, 330);

  /* CHECKING AND FLUSHING */
  printf("line \n");
  fflush(stdout);
}

void circle_cb(){
  /* POPULATE THE SUBWINDOW */
  clear_cb();
  subwindow->begin();
  /* spline */
  fltk::ValueInput *splinepoints = new fltk::ValueInput(200, 10, 100, 20, "N. points for spline aprox.: ");
  splinepoints->range(0, 10000);
  splinepoints->linesize(1.0);
  splinepoints->step(1);
  inputindex[0] = subwindow->find(splinepoints);
  /* radius */
  fltk::ValueInput *radius = new fltk::ValueInput(200, 30, 100, 20, "Insert radius: ");
  radius->range(-1000, 1000);
  radius->step(0.01);
  inputindex[1] = subwindow->find(radius);
  /* center x */
  fltk::ValueInput *centerx = new fltk::ValueInput(200, 50, 100, 20, "Center abscissa (x): ");
  centerx->range(-1000, 1000);
  centerx->step(0.01);
  inputindex[2] = subwindow->find(centerx);
  /* center y */
  fltk::ValueInput *centery = new fltk::ValueInput(450, 50, 100, 20, "Center ordinata (y): ");
  centery->range(-1000, 1000);
  centery->step(0.01);
  inputindex[3] = subwindow->find(centery);
  /* start arc */
  fltk::ValueInput *sarc = new fltk::ValueInput(200, 70, 100, 20, "Start angle: ");
  sarc->range(-360, 360);
  sarc->linesize(1.0);
  sarc->step(1);
  inputindex[4] = subwindow->find(sarc);
  /* end arc */
  fltk::ValueInput *earc = new fltk::ValueInput(450, 70, 100, 20, "End angle: ");
  earc->range(-360, 360);
  earc->linesize(1.0);
  earc->step(1);
  inputindex[5] = subwindow->find(earc);
  /* [i|s|h]data */
  fltk::Button *new_dataline = new fltk::Button(0, 100, 580, 20, "Press here to add a new data entry");
  new_dataline->callback((fltk::Callback *)new_dataline_cb);
  datagroup = new fltk::ScrollGroup(0, 140, 580, 140, "Insert [i|s|h]data here:");
  datagroup->scrollbar.linesize(20);
  /* buttons */
  fltk::Button *cancel = new fltk::Button(200, 290, 100, 20, "Cancel");
  fltk::Button *add = new fltk::Button(310, 290, 100, 20, "Add");
  cancel->callback((fltk::Callback *)clear_cb);
  add->callback((fltk::Callback *)add_circle_cb);
  subwindow->end();
  subwindow->redraw(0, 0, 620, 380);

  /* CHECKING AND FLUSHING */
  printf("circle\n");
  fflush(stdout);
}

void ellipse_cb(){
  /* POPULATE THE SUBWINDOW */
  clear_cb();
  subwindow->begin();
  /* spline */
  fltk::ValueInput *splinepoints = new fltk::ValueInput(200, 10, 100, 20, "N. points for spline aprox.: ");
  splinepoints->range(0, 10000);
  splinepoints->linesize(1.0);
  splinepoints->step(1);
  inputindex[0] = subwindow->find(splinepoints);
  /* semix */
  fltk::ValueInput *semix = new fltk::ValueInput(200, 30, 100, 20, "Insert x semiaxis length: ");
  semix->range(-1000, 1000);
  semix->step(0.01);
  inputindex[1] = subwindow->find(semix);
  /* semiy */
  fltk::ValueInput *semiy = new fltk::ValueInput(450, 30, 100, 20, "Insert y semiaxis length: ");
  semiy->range(-1000, 1000);
  semiy->step(0.01);
  inputindex[2] = subwindow->find(semiy);
  /* center x */
  fltk::ValueInput *centerx = new fltk::ValueInput(200, 50, 100, 20, "Center abscissa (x): ");
  centerx->range(-1000, 1000);
  centerx->step(0.01);
  inputindex[3] = subwindow->find(centerx);
  /* center y */
  fltk::ValueInput *centery = new fltk::ValueInput(450, 50, 100, 20, "Center ordinata (y): ");
  centery->range(-1000, 1000);
  centery->step(0.01);
  inputindex[4] = subwindow->find(centery);
  /* start arc */
  fltk::ValueInput *sarc = new fltk::ValueInput(200, 70, 100, 20, "Start angle: ");
  sarc->range(-360, 360);
  sarc->linesize(1.0);
  sarc->step(1);
  inputindex[5] = subwindow->find(sarc);
  /* end arc */
  fltk::ValueInput *earc = new fltk::ValueInput(450, 70, 100, 20, "End angle: ");
  earc->range(-360, 360);
  earc->linesize(1.0);
  earc->step(1);
  inputindex[6] = subwindow->find(earc);
  /* [i|s|h]data */
  fltk::Button *new_dataline = new fltk::Button(0, 100, 580, 20, "Press here to add a new data entry");
  new_dataline->callback((fltk::Callback *)new_dataline_cb);
  datagroup = new fltk::ScrollGroup(0, 140, 580, 140, "Insert [i|s|h]data here:");
  datagroup->scrollbar.linesize(20);
  /* buttons */
  fltk::Button *cancel = new fltk::Button(200, 290, 100, 20, "Cancel");
  fltk::Button *add = new fltk::Button(310, 290, 100, 20, "Add");
  cancel->callback((fltk::Callback *)clear_cb);
  add->callback((fltk::Callback *)add_ellipse_cb);
  subwindow->end();
  subwindow->redraw(0, 0, 620, 380);

  /* CHECKING AND FLUSHING */
  printf("ellipse\n");
  fflush(stdout);
}

void new_dataline_cb() {
  int currentpos;
  datagroup->begin();
  /* This is a hard hack since inputs (and any other thing) got created 
   * from actual origin and not from widget (datagroup inn this case)
   * absolute position, anyway the first time the input got creted with
   * the bar fully scored and unexpected 10 pixel offset is generated.
   * I suspect this is an initialization bug in fltk code
   * I fugured out the problem and resoleved it, it is a bad, but not
   * buggy, behavor or fltk. It just wipesout white space if not filled
   * by some whidget (window or anything) */
  if(datagroup->yposition() < 0) {
    currentpos = ydataposition;
  }
  else {
    currentpos = ydataposition - datagroup->yposition();
  }
  //  fprintf(stdout, "yposition(): %i\n", datagroup->yposition());
  //  fprintf(stdout, "currentpos: %i\n", currentpos);
  //  fflush(stdout);
  //  ydataposition = ydataposition - datagroup->yposition();
  fltk::ValueInput *idata = new fltk::ValueInput(50, currentpos, 100, 20, "idata: ");
  idata->range(0, 3);
  idata->linesize(1.0);
  fltk::ValueInput *sdata = new fltk::ValueInput(250, currentpos, 100, 20, "sdata: ");
  sdata->range(-1000, 1000);
  sdata->linesize(0.01);
  fltk::ValueInput *hdata = new fltk::ValueInput(450, currentpos, 100, 20, "hdata: ");
  hdata->range(-1000, 1000);
  hdata->linesize(0.01);
  datagroup->end();
  datagroup->redraw(0, currentpos, 620, 20);  // maybe this is not anymore necessary, but should be correct.
  ydataposition = ydataposition + 20;
}

void clear_cb(){
  ydataposition = 0;
  for(int i = 0; i < 10; i++) inputindex[i] = 0;
  subwindow->clear();
  subwindow->redraw(0, 0, 620, 330);
}

void add_line_cb() {
  int ndata;
  int validdata = 0;
  Line *prim = (Line *)(bdfile->add_prim(PR_LINE));
  //  printf("subwindow ha %i figli\n", subwindow->children());
  prim->spline = (int)((fltk::ValueInput *)subwindow->child(inputindex[0]))->value();
  prim->x1 = ((fltk::ValueInput *)subwindow->child(inputindex[1]))->value();
  prim->y1 = ((fltk::ValueInput *)subwindow->child(inputindex[2]))->value();
  prim->x2 = ((fltk::ValueInput *)subwindow->child(inputindex[3]))->value();
  prim->y2 = ((fltk::ValueInput *)subwindow->child(inputindex[4]))->value();
  printf("Spline: %i x1: %f y1: %f x2: %f y2: %f\n", prim->spline, prim->x1, prim->y1, prim->x2, prim->y2);
  fflush(stdout);
  ndata = datagroup->children();
  prim->allocatedata(ndata/3);
  /* this could be done nicer with k%3 as second argument and some
   * other arragnements, but it's ok for now
   */
  for(int k = 0; k < ndata; k = k + 3) {
    if(((fltk::ValueInput *)datagroup->child(k))->value() > 0) {
      fprintf(stdout, "k/3: %i child n.: %i valid data (x): %i\n", k/3, k, validdata);
      fflush(stdout);
      prim->setdata(validdata, 0, ((fltk::ValueInput *)datagroup->child(k))->value());
      prim->setdata(validdata, 1, ((fltk::ValueInput *)datagroup->child(k + 1))->value());
      prim->setdata(validdata, 2, ((fltk::ValueInput *)datagroup->child(k + 2))->value());
      validdata++;
      prim->curdata++;
    }
  }
#ifdef DEBUGGING
  fprintf(stdout, "calling setprim\n");
#endif
  mygl->setprim(bdfile->firstprimitive, bdfile->elements);
  mygl->redraw();
}

void add_circle_cb() {
  int ndata;
  int validdata = 0;
  Circle *prim = (Circle *)(bdfile->add_prim(PR_CIRCLE));
  printf("subwindow ha %i figli\n", subwindow->children());
  prim->spline = (int)((fltk::ValueInput *)subwindow->child(inputindex[0]))->value();
  prim->radius = ((fltk::ValueInput *)subwindow->child(inputindex[1]))->value();
  prim->centerx = ((fltk::ValueInput *)subwindow->child(inputindex[2]))->value();
  prim->centery = ((fltk::ValueInput *)subwindow->child(inputindex[3]))->value();
  prim->startarc = (int)((fltk::ValueInput *)subwindow->child(inputindex[4]))->value();
  prim->endarc = (int)((fltk::ValueInput *)subwindow->child(inputindex[5]))->value();
  printf("Spline: %i radius: %f centerx: %f centery: %f startarc: %i endarc: %i\n", prim->spline, prim->radius, prim->centerx, prim->centery, prim->startarc, prim->endarc);
  fflush(stdout);
  ndata = datagroup->children();
  prim->allocatedata(ndata/3);
  /* this could be done nicer with k%3 as second argument and some
   * other arragnements, but it's ok for now
   */
  for(int k = 0; k < ndata; k = k + 3) {
    if(((fltk::ValueInput *)datagroup->child(k))->value() > 0) {
      fprintf(stdout, "k/3: %i child n.: %i valid data (x): %i\n", k/3, k, validdata);
      fflush(stdout);
      prim->setdata(validdata, 0, ((fltk::ValueInput *)datagroup->child(k))->value());
      prim->setdata(validdata, 1, ((fltk::ValueInput *)datagroup->child(k + 1))->value());
      prim->setdata(validdata, 2, ((fltk::ValueInput *)datagroup->child(k + 2))->value());
      validdata++;
      prim->curdata++;
    }
  }
#ifdef DEBUGGING
  fprintf(stdout, "calling setprim\n");
#endif
  mygl->setprim(bdfile->firstprimitive, bdfile->elements);
  mygl->redraw();
}

void add_ellipse_cb() {
  int ndata;
  int validdata = 0;
  Ellipse *prim = (Ellipse *)(bdfile->add_prim(PR_ELLIPSE));
  printf("subwindow ha %i figli\n", subwindow->children());
  prim->spline = (int)((fltk::ValueInput *)subwindow->child(inputindex[0]))->value();
  prim->semix = ((fltk::ValueInput *)subwindow->child(inputindex[1]))->value();
  prim->semiy = ((fltk::ValueInput *)subwindow->child(inputindex[2]))->value();
  prim->centerx = ((fltk::ValueInput *)subwindow->child(inputindex[3]))->value();
  prim->centery = ((fltk::ValueInput *)subwindow->child(inputindex[4]))->value();
  prim->startarc = (int)((fltk::ValueInput *)subwindow->child(inputindex[5]))->value();
  prim->endarc = (int)((fltk::ValueInput *)subwindow->child(inputindex[6]))->value();
  printf("Spline: %i semix: %f semiy: %f centerx: %f centery: %f startarc: %i endarc: %i\n", prim->spline, prim->semix, prim->semiy, prim->centerx, prim->centery, prim->startarc, prim->endarc);
  fflush(stdout);
  ndata = datagroup->children();
  prim->allocatedata(ndata/3);
  /* this could be done nicer with k%3 as second argument and some
   * other arragnements, but it's ok for now
   */
  for(int k = 0; k < ndata; k = k + 3) {
    if(((fltk::ValueInput *)datagroup->child(k))->value() > 0) {
      fprintf(stdout, "k/3: %i child n.: %i valid data (x): %i\n", k/3, k, validdata);
      fflush(stdout);
      prim->setdata(validdata, 0, ((fltk::ValueInput *)datagroup->child(k))->value());
      prim->setdata(validdata, 1, ((fltk::ValueInput *)datagroup->child(k + 1))->value());
      prim->setdata(validdata, 2, ((fltk::ValueInput *)datagroup->child(k + 2))->value());
      validdata++;
      prim->curdata++;
    }
  }
#ifdef DEBUGGING
  fprintf(stdout, "calling setprim\n");
#endif
  mygl->setprim(bdfile->firstprimitive, bdfile->elements);
  mygl->redraw();
}

void about_cb(){
  fltk::message("%s by Gabriele Dini Ciacci - version %s\n", PACKAGE, VERSION);
}

void glview_cb(){
  if(mygl->visible()){
    mygl->hide();
  }
  else {
    mygl->show();
  }
}
